from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class Dormitory(Base):
    """
    Модель "Общежитие"
    """
    __tablename__ = 'platonus_univer_dormitories'

    id = Column('dormitoryID', Integer, primary_key=True)  # Идентификатор
    address = Column(String(256))  # Адрес общежития
    status = Column(Integer)  # Статус

    def __repr__(self):
        return '<Dormitory (id={} address={} status={})>'.format(
            self.id, self.address, self.status
        )
    
    def __str__(self):
        return self.address


class DormitoryRoom(Base):
    """
    Модель "Комната общежития"
    """
    __tablename__ = 'univer_dormitory_rooms'

    id = Column('dormitory_room_id', Integer, primary_key=True)  # Идентификатор
    dormitory_id = Column(ForeignKey('platonus_univer_dormitories.dormitoryID'))
    dormitory = relationship('Dormitory')  # Общежитие
    floor = Column('dormitory_room_floor', Integer)  # Этаж
    size = Column('dormitory_room_size', Integer)  # Вместимость
    number_ru = Column('dormitory_room_number_ru', String(30))  # Наименование комнаты (на русском)
    number_kz = Column('dormitory_room_number_kz', String(30))  # Наименование комнаты (на казахском)
    number_en = Column('dormitory_room_number_en', String(30))  # Наименование комнаты (на английском)
    status = Column(Integer)  # Статус

    def __repr__(self):
        return '<DormitoryRoom {} (id={} dormitory_id={} floor={} size={} status={}>'.format(self, self.id, self.dormitory_id, self.floor, self.size, self.status)
    
    def __str__(self):
        return self.number_ru


class DormitoryApplicant(Base):
    """
    Модель "Заявление общежития"
    """
    __tablename__ = 'univer_dormitory_applicant'

    id = Column('dormitory_applicant_id', Integer, primary_key=True)  # Идентификатор
    applicant_create = Column('dormitory_applicant_create', DateTime)  # Дата и время создания заявления
    applicant_initiator = Column('dormitory_applicant_initiator', Integer)  # Инициатор заявки
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')  # Факультет
    dormitory_id = Column(ForeignKey('platonus_univer_dormitories.dormitoryID'))
    dormitory = relationship('Dormitory')  # Общежитие
    applicant_status = Column('dormitory_applicant_status', Integer)  # Статус заявки
    status = Column(Integer)  # Статус
    year = Column(Integer)  # Год

    def __repr__(self):
        return '<DormitoryApplicant (id={}, applicant_create={}, dormitory_id={}, applicant_status={}, year={})>'.format(
            self.id, self.applicant_create, self.dormitory_id, self.applicant_status, self.year
        )


class DormitoryApplicantStudent(Base):
    """
    Модель "Студент в заявке общежития"
    """
    __tablename__ = 'univer_dormitory_applicant_students'

    applicant_id = Column('dormitory_applicant_id', ForeignKey('univer_dormitory_applicant.dormitory_applicant_id'), primary_key=True)
    applicant = relationship('DormitoryApplicant')  # Заявление общежития
    student_id = Column(ForeignKey('univer_students.students_id'), primary_key=True)
    student = relationship('Student')  # Студент
    student_status = Column('dormitory_applicant_student_status', Integer)  # Статус студента в заявке общежития
    date_begin = Column('dormitory_student_date_begin', DateTime)  # Дата и время начала
    date_end = Column('dormitory_student_date_end', DateTime)  # Дата и время окончания
    created_at = Column('dormitory_applicant_student_create', DateTime)  # Дата и время создания
    updated_at = Column('dormitory_applicant_student_update', DateTime)  # Дата и время обновления
    status = Column(Integer)  # Статус

    def __repr__(self):
        return '<DormitoryApplicantStudent (dormitory_applicant_id={} student_id={} student_status={})>'.format(
            self.applicant_id, self.student_id, self.student_status
        )


class StudentDormitoryRoomLink(Base):
    """
    Модель "Связь между студентом и комнатой"
    """
    __tablename__ = 'univer_student_dormitory_room_link'

    student_id = Column(ForeignKey('univer_students.students_id'), primary_key=True)
    student = relationship('Student')  # Студент
    dormitory_room_id = Column(ForeignKey('univer_dormitory_rooms.dormitory_room_id'), primary_key=True)
    dormitory_room = relationship('DormitoryRoom')  # Комната общежития
    year = Column(Integer)  # Год
    created_at = Column('created', DateTime)  # Дата и время создания
    applicant_id = Column('dormitory_applicant_id', ForeignKey('univer_dormitory_applicant.dormitory_applicant_id'))
    applicant = relationship('DormitoryApplicant')  # Заявления общежития

    def __repr__(self):
        return '<StudentDormitoryRoomLink (student_id={} dormitory_room_id={} year={} applicant_id={})>'.format(
            self.student_id, self.dormitory_room_id, self.year, self.applicant_id
        )


class StudentDormitoryRoomSettleHistory(Base):
    """
    Модель "История проживания в комнате общежития студентом"
    """
    __tablename__ = 'univer_student_dormitory_room_settle_history'

    id = Column('history_id', Integer, primary_key=True)  # Идентификатор
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')  # Студент
    dormitory_room_id = Column(ForeignKey('univer_dormitory_rooms.dormitory_room_id'))
    dormitory_room = relationship('DormitoryRoom')  # Комната общежития
    date = Column('history_date', DateTime)  # Дата и время
    user_id = Column(ForeignKey('univer_users.user_id'))
    user = relationship('User')  # Пользователь
    personnel_id = Column('personal_id', ForeignKey('univer_personal.personal_id'))
    personnel = relationship('Personnel')  # Сотрудник
    applicant_id = Column('dormitory_applicant_id', ForeignKey('univer_dormitory_applicant.dormitory_applicant_id'))
    applicant = relationship('DormitoryApplicant')  # Заявления общежития
    dormitory_id = Column(ForeignKey('platonus_univer_dormitories.dormitoryID'))
    dormitory = relationship('Dormitory')  # Общежитие
    order_type = Column(Integer)  # Тип приказа
    order_date = Column(DateTime)  # Дата и время приказа
    order_status = Column(Integer)  # Статус приказа
    dormitory_room_size = Column('dormitory_room_size', Integer)  # Размер комнаты общежития

    def __repr__(self):
        return '<StudentDormitoryRoomSettleHistory (id={} student_id={} dormitory_room_id={} date={} applicant_id={} order_type={}'.format(
            self.id,
            self.student_id,
            self.dormitory_room_id,
            self.date,
            self.applicant_id,
            self.order_type
        )


class DormitoryStudentAccess(Base):
    """
    Модель "Доступ студента к общежитию"
    """
    __tablename__ = 'univer_dormitory_student_access'

    student_id = Column(ForeignKey('univer_students.students_id'), primary_key=True)
    student = relationship('Student')  # Студент
    status = Column('access_status', Integer)  # Статус доступа
    year = Column(Integer)  # Год

    def __repr__(self):
        return '<DormitoryStudentAccess (student_id={}, status={}, year={})>'.format(
            self.student_id,
            self.status,
            self.year
        )


class DRIVER:
    FREETDS = 'FreeTDS'
    ODBC_17 = 'ODBC+Driver+17+for+SQL+Server'
    NATIVE_11 = 'SQL+Server+Native+Client+11.0'

class Config:
    def __init__(self, host, user, password, db='univer', driver=DRIVER.FREETDS):
        self.host = host
        self.user = user
        self.password = password
        self.db = db
        self.driver = driver


from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class Speciality(Base):
    """
    Модель "Специальность"
    """

    __tablename__ = 'univer_speciality'

    id = Column('speciality_id', Integer, primary_key=True)
    status = Column(Integer)
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')
    stage_id = Column(ForeignKey('univer_stage.stage_id'))
    stage = relationship('Stage')
    name_kz = Column('speciality_name_kz', String(200))
    name_ru = Column('speciality_name_ru', String(200))
    name_en = Column('speciality_name_en', String(200))
    code = Column('speciality_okpd', String(10))
    description_kz = Column('speciality_discription_kz', String)
    description_ru = Column('speciality_discription_ru', String)
    description_en = Column('speciality_discription_en', String)
    result_ru = Column(String)
    result_kz = Column(String)
    result_en = Column(String)
    type = Column(Integer)

    def __repr__(self):
        return "<Speciality {}>".format(self)

    def __str__(self):
        return "{} - {}".format(self.code, self.name_ru)


class SpecialityChair(Base):
    """
    Модель отношений "Специальность-Кафедра"
    """

    __tablename__ = 'univer_speciality_chair'

    speciality_id = Column(ForeignKey(
        'univer_speciality.speciality_id'), primary_key=True)
    speciality = relationship('Speciality')
    chair_id = Column(ForeignKey('univer_chair.chair_id'), primary_key=True)
    chair = relationship('Chair')

    def __repr__(self):
        return '<SpecialityChair {}>'.format(self)

    def __str__(self):
        return '{} - {}'.format(self.chair, self.speciality)


class Subject(Base):
    """
    Модель "Дисциплина"
    """

    __tablename__ = 'univer_subject'

    id = Column('subject_id', Integer, primary_key=True)
    name_kz = Column('subject_name_kz', String(500))
    name_ru = Column('subject_name_ru', String(500))
    name_en = Column('subject_name_en', String(500))
    description_kz = Column('subject_description_kz', String)
    description_ru = Column('subject_description_ru', String)
    description_en = Column('subject_description_en', String)
    status = Column(Integer)

    def __repr__(self):
        return '<Subject {}>'.format(self.name_ru)

    def __str__(self):
        return self.name_ru


class EducType(Base):
    __tablename__ = 'univer_educ_type'

    id = Column('educ_type_id', Integer, primary_key=True)
    name_ru = Column('educ_type_name_ru', String(100))

    def __repr__(self):
        return '<EducType {}>'.format(self.name_ru)

    def __str__(self):
        return self.name_ru


class LangDivision(Base):
    """
    Модель "Языковый отдел"
    """
    __tablename__ = 'univer_lang_division'

    # Идентификатор
    id = Column('lang_division_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('lang_division_name_ru', String(100))
    name_kz = Column('lang_division_name_kz', String(100))
    name_en = Column('lang_division_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<LangDivision {} (id={}, status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class DocumentIdentity(Base):
    """
    Модель "Тип документа"
    Статус: Выполнено
    """

    __tablename__ = 'univer_document_identity'

    id = Column('document_identity_type', Integer, primary_key=True)
    name_ru = Column('document_name_ru', String(100))
    name_kz = Column('document_name_kz', String(100))
    name_en = Column('document_name_en', String(100))

    def __repr__(self):
        return '<DocumentIdentity {} (id={})'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class Institution(Base):
    """
    Модель "Учебное заведение"
    Статус: Выполняется
    """

    __tablename__ = 'univer_edu_institutions'

    id = Column('edu_institution_id', Integer, primary_key=True)
    name_kz = Column('edu_institution_name_kz', String)
    name_ru = Column('edu_institution_name_ru', String)
    name_en = Column('edu_institution_name_en', String)

    def __repr__(self):
        return '<Institution {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class GraduateDocType(Base):
    """
    Тип документа об окончании учебного заведения перед поступлением в университет
    Статус: Выполняется
    """

    __tablename__ = 'univer_graduate_doctypes'

    id = Column('graduate_doctype_id', Integer, primary_key=True)
    name_ru = Column('graduate_doctype_name_ru', String)
    name_kz = Column('graduate_doctype_name_kz', String)
    name_en = Column('graduate_doctype_name_en', String)

    def __repr__(self):
        return '<GraduateDocType {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class GraduateInfo(Base):
    """
    Данные об окончании учебного заведения перед поступлением в университет
    Статус: Выполняется
    """

    __tablename__ = 'univer_graduate_info'

    id = Column('graduate_info_id', Integer, primary_key=True)
    date = Column('graduate_info_date', DateTime)
    institution_name = Column('graduate_info_institution_name', String)
    series = Column('graduate_info_series', String)
    number = Column('graduate_info_number', String)
    graduate_doctype_id = Column(ForeignKey(
        'univer_graduate_doctypes.graduate_doctype_id'))
    graduate_doctype = relationship('GraduateDocType')
    institution_id = Column('edu_institution_id', ForeignKey(
        'univer_edu_institutions.edu_institution_id'))
    institution = relationship('Institution')

    def __repr__(self):
        return '<Graduate {} (id={})>'.format(self, self.id)

    def __str__(self):
        return '{} {}'.format(self.series, self.number)


class Contract(Base):
    """
    Модель "Договор студента"
    Статус: Выполняется
    """

    __tablename__ = 'univer_contract'

    id = Column('contract_id', Integer, primary_key=True)
    number = Column('contract_number', String)
    date_received = Column('contract_date_recieved', DateTime)
    status = Column(Integer)
    student_id = Column('students_id', ForeignKey(
        'univer_students.students_id'))
    student = relationship('Student')

    def __repr__(self):
        return '<Contract {} (id={})>'.format(self, self.id)

    def __str__(self):
        '{} ({})'.format(self.student, self.number)


class AcademCalendar(Base):
    """
    Модель "Академический календарь"
    """

    __tablename__ = 'univer_academ_calendar_pos'

    id = Column('acpos_id', Integer, primary_key=True)
    educ_plan_id = Column(ForeignKey('univer_educ_plan.educ_plan_id'))
    educ_plan = relationship('EducPlan')
    acpos_semester = Column(Integer)
    acpos_module = Column(Integer)
    controll_id = Column('control_id', ForeignKey('univer_control.control_id'))
    controll = relationship('Controll')
    acpos_weeks = Column(Integer)
    acpos_date_start = Column(DateTime)
    acpos_date_end = Column(DateTime)

    def __repr__(self):
        return '<AcademCalendar {}>'.format(self)

    def __str__(self):
        return '{}'.format(self.id)


class EducPlan(Base):
    """
    Учебный план
    """
    __tablename__ = 'univer_educ_plan'

    # Идентификатор
    id = Column('educ_plan_id', Integer, primary_key=True)

    # Специальность
    speciality_id = Column('speciality_id', ForeignKey(
        'univer_speciality.speciality_id'))
    speciality = relationship('Speciality')

    # Форма обучения
    education_form_id = Column(ForeignKey(
        'univer_education_form.education_form_id'))
    education_form = relationship('EducationForm')

    # Уровень образования
    edu_level_id = Column(ForeignKey('univer_edu_levels.edu_level_id'))
    edu_level = relationship('EduLevel')

    # Учебный год
    year = Column('educ_plan_adm_year', Integer)

    # Количество семестров в учебном плане
    semesters = Column('educ_plan_number_of_semestr', Integer)

    # Количество обязательных кредитов
    min_cred_total = Column('educ_plan_min_cred_total', Integer)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return "<EducPlan {} {}>".format(self.speciality, self.year)

    def __str__(self):
        return "{} {}".format(self.speciality, self.year)


class EducPlanPos(Base):
    """
    Модель "Позиция учебного плана"
    """
    __tablename__ = 'univer_educ_plan_pos'

    # Идентификатор
    id = Column('educ_plan_pos_id', Integer, primary_key=True)

    # Учебный план
    educ_plan_id = Column('educ_plan_id', ForeignKey(
        'univer_educ_plan.educ_plan_id'))
    educ_plan = relationship('EducPlan')

    # Код дисциплины
    code = Column('rup_ru', String(50))

    # Дисциплина
    subject_id = Column('subject_id', ForeignKey('univer_subject.subject_id'))
    subject = relationship('Subject')

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'))
    controll_type = relationship('ControllType')

    # Кредит
    credit = Column('educ_plan_pos_credit', Integer)

    # Семестр
    semester = Column('educ_plan_pos_semestr', Integer)

    # Цикл
    cycle_id = Column(ForeignKey('univer_cycles.cycle_id'))
    cycle = relationship('Cycle')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return "<EducPlanPos {}: {} ({} семестр)>".format(self.educ_plan, self.subject, self.semester)

    def __str__(self):
        return "{}: {} ({} семестр)".format(self.educ_plan, self.subject, self.semester)


class Cycle(Base):
    """
    Модель "Цикл"
    """
    __tablename__ = 'univer_cycles'

    # Идентификатор
    id = Column('cycle_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('cycle_name_ru', String)
    name_kz = Column('cycle_name_kz', String)
    name_en = Column('cycle_name_en', String)

    def __repr__(self):
        return '<Cycle {}>'.format(self)

    def __str__(self):
        return self.name_ru


class Attendance(Base):
    """
    Модель "Журнал посещаемости"
    """

    __tablename__ = 'univer_attendance'

    date = Column('att_date', Date, primary_key=True)
    grade = Column('ball', Float)
    was = Column(Boolean)
    student_id = Column(ForeignKey(
        'univer_students.students_id'), primary_key=True)
    student = relationship('Student')
    group_id = Column(ForeignKey('univer_group.group_id'), primary_key=True)
    group = relationship('Group')

    def __repr__(self):
        return '<Attendance {}: {} балл ({})>'.format(self.student, self.grade, self.date)

    def __str__(self):
        return '{}: {} балл ({})'.format(self.student, self.grade, self.date)


class EducLang(Base):
    """
    Модель "Язык обучения"
    """
    __tablename__ = 'univer_educ_lang'

    # Идентификатор
    id = Column('educ_lang_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('educ_lang_name_ru', String)
    name_kz = Column('educ_lang_name_kz', String)
    name_en = Column('educ_lang_name_en', String)

    # Короткое наименование
    short_name_ru = Column('educ_lang_short_name_ru', String)
    short_name_kz = Column('educ_lang_short_name_kz', String)
    short_name_en = Column('educ_lang_short_name_en', String)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<EducLang {}>'.format(self)

    def __str__(self):
        return self.name_ru


class Group(Base):
    """
    Модель "Группа"
    Статус: Выполняется
    """

    __tablename__ = 'univer_group'

    id = Column('group_id', Integer, primary_key=True)
    educ_plan_pos_id = Column('educ_plan_pos_id', ForeignKey(
        'univer_educ_plan_pos.educ_plan_pos_id'))
    educ_plan_pos = relationship('EducPlanPos', backref='groups')
    teacher_id = Column('teacher_id', ForeignKey('univer_teacher.teacher_id'))
    teacher = relationship('Teacher')
    year = Column('group_year', Float)
    semester = Column('group_semestr', Integer)
    acpos_module = Column('acpos_module', Integer)
    educ_type_id = Column('educ_type_id', ForeignKey(
        'univer_educ_type.educ_type_id'))
    educ_type = relationship('EducType')
    lang_division_id = Column('lang_division_id', ForeignKey(
        'univer_lang_division.lang_division_id'))
    lang_division = relationship('LangDivision')

    # Язык обучения
    educ_lang_id = Column(ForeignKey('univer_educ_lang.educ_lang_id'))
    educ_lang = relationship('EducLang')

    # Семестр повторного обучения
    retake_semester = Column('group_retake_semestr', Integer)

    def __repr__(self):
        return '<Group {} (id={} educ_plan_pos_id={} teacher_id={})>'.format(self, self.id, self.educ_plan_pos_id, self.teacher_id)

    def __str__(self):
        return '{} ({} год)'.format(self.educ_plan_pos.educ_plan.speciality, self.educ_plan_pos.educ_plan.year)


class GroupStudent(Base):
    """
    Модель "Студент в группе"
    """

    __tablename__ = 'univer_group_student'

    id = Column('group_student_id', Integer, primary_key=True)
    group_id = Column(ForeignKey('univer_group.group_id'))
    group = relationship('Group', backref='group_students')
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')


class Controll(Base):
    """
    Модель "Контроль"
    """

    __tablename__ = 'univer_control'

    # Идентификатор
    id = Column('control_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('control_name_ru', String(100))
    name_kz = Column('control_name_kz', String(100))
    name_en = Column('control_name_en', String(100))

    def __repr__(self):
        return '<Controll {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class ControllType(Base):
    """
    Модель "Тип контроля"
    """

    __tablename__ = 'univer_controll_type'

    # Идентификатор
    id = Column('controll_type_id', Integer, primary_key=True)

    # Наименование
    name_kz = Column('controll_type_name_kz', String(100))
    name_ru = Column('controll_type_name_ru', String(100))
    name_en = Column('controll_type_name_en', String(100))

    def __repr__(self):
        return '<ControllType {}>'.format(self)

    def __str__(self):
        return self.name_ru


class ControllTypeControllLink(Base):
    """
    Модель "Связь между Controll и ControllType
    Статус: Выполняется
    """

    __tablename__ = 'univer_controll_type_control_link'

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'), primary_key=True)
    controll_type = relationship('ControllType')

    # Контроль
    controll_id = Column('control_id', ForeignKey(
        'univer_control.control_id'), primary_key=True)
    controll = relationship('Controll')

    # Тип ведомости
    sheet_type_id = Column(ForeignKey(
        'univer_sheet_type.sheet_type_id'), primary_key=True)
    sheet_type = relationship('SheetType')

    def __repr__(self):
        return '<ControllTypeControllLink {} (controll_type_id={} controll_id={} sheet_type_id={}>'.format(self, self.controll_type_id, self.controll_id, self.sheet_type_id)

    def __str__(self):
        return '{}-{}'.format(self.controll_type, self.controll)


class MarkType(Base):
    """
    Модель "Тип оценки"
    """

    __tablename__ = 'univer_mark_type'

    # Идентификатор
    id = Column('mark_type_id', Integer, primary_key=True)

    # Символ оценки
    symbol = Column('mark_type_symbol', String(10))

    # Минимальное значение
    min_val = Column('mark_type_minval', Integer)

    # Максимальное значение
    max_val = Column('mark_type_maxval', Integer)

    # GPA
    gpa = Column('mark_type_gpa', Float)

    # ???
    arg = Column('mark_type_arg', Integer)

    def __repr__(self):
        return '<MarkType {}>'.format(self)

    def __str__(self):
        return '{} ({})'.format(self.symbol, self.gpa)


class Progress(Base):
    """
    Прогресс студента
    """
    __tablename__ = 'univer_progress'

    # Идентификатор
    id = Column('progress_id', Integer, primary_key=True)

    # Академический год
    academ_year = Column(Integer)

    # Академический семестр
    academ_semester = Column('semestr', Integer)

    # Студент
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')

    # Дисциплина
    subject_id = Column(ForeignKey('univer_subject.subject_id'))
    subject = relationship('Subject')

    # Тип дисциплины
    subject_type = Column(Integer)

    # Наименование дисциплины
    subject_name_ru = Column(String(500))
    subject_name_kz = Column(String(500))
    subject_name_en = Column(String(500))

    # Тип оценки
    mark_type_id = Column(ForeignKey('univer_mark_type.mark_type_id'))
    mark_type = relationship('MarkType')

    # Кредит
    credit = Column('progress_credit', Integer)

    # Оценки
    result_rk1 = Column('progress_result_rk1', Integer)
    result_rk2 = Column('progress_result_rk2', Integer)
    result = Column('progress_result', Integer)

    # Семестры
    semester = Column('n_seme', Integer)

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'))
    controll_type = relationship('ControllType')

    # Часы
    hours = Column('progress_hours', Integer)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<Progress {}>'.format(self)

    def __str__(self):
        return '{} - {}'.format(self.student, self.subject)


class Country(Base):
    __tablename__ = 'univer_country'

    id = Column('country_id', Integer, primary_key=True)
    status = Column(Integer)
    name_ru = Column('country_name_ru', String(500))
    name_kz = Column('country_name_kz', String(500))
    name_en = Column('country_name_en', String(500))
    code = Column('country_code', Integer)
    current = Column('country_current', Integer)
    letter_code = Column('country_letter_code', String(10))
    alfa3_code = Column(String(5))

    def __repr__(self):
        return '<Country {}>'.format(self)

    def __str__(self):
        return self.name_ru



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class Order(Base):
    """
    Приказ
    """
    __tablename__ = 'univer_order'

    # Идентификатор
    id = Column('order_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Тип приказа
    order_type_id = Column(ForeignKey('univer_order_type.order_type_id'))
    order_type = relationship('OrderType')

    # Действие приказа
    order_action_id = Column(ForeignKey('univer_order_action.order_action_id'))
    order_action = relationship('OrderAction')

    # Номер приказа
    number = Column('order_number', String(20))

    # Дата приказа
    date = Column('order_date', DateTime)

    # Данные приказа в XML-формате
    data = Column('order_data', String)

    # Год
    year = Column('order_year', Integer)

    # Семестр
    semester = Column('order_semester', Integer)

    # Одобрен
    approved = Column('order_approved', Integer)

    # Подписан (???)
    signed = Column('order_signed', Integer)

    # Дата и время создания
    date_create = Column('order_date_create', DateTime)

    # Дата и время одобрения
    date_approve = Column('order_date_approve', DateTime)

    # Дата и время окончания
    date_done = Column('order_date_done', DateTime)

    # Когда процесс окончания начался
    done_process_start = Column('order_done_process_start', DateTime)

    # Расположение сгенерированного документа
    document = Column('order_document', String(50))

    # Ступень обучения
    stage_id = Column(ForeignKey('univer_stage.stage_id'))
    stage = relationship('Stage')

    # Форма обучения
    education_form_id = Column(ForeignKey(
        'univer_education_form.education_form_id'))
    education_form = relationship('EducationForm')

    # Тип оплаты
    payment_form_id = Column(ForeignKey(
        'univer_payment_forms.payment_form_id'))
    payment_form = relationship('PaymentForm')

    # Уровень обучения
    edu_level_id = Column(ForeignKey('univer_edu_levels.edu_level_id'))
    edu_level = relationship('EduLevel')

    def __repr__(self):
        return '<Order {} (id={} status={} order_type={} stage={} education_form={} payment_form={} edu_level={}'.format(
            self,
            self.id,
            self.status,
            self.order_type_id,
            self.stage_id,
            self.education_form_id,
            self.payment_form_id,
            self.edu_level_id
        )

    def __str__(self):
        return '{} - {}'.format(self.number, self.date)


class OrderType(Base):
    """
    Модель "Типы приказов"
    Статус: Выполняется
    """

    __tablename__ = 'univer_order_type'

    # Идентификатор
    id = Column('order_type_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Наименование
    name_ru = Column('order_type_name_ru', String(300))
    name_kz = Column('order_type_name_kz', String(300))
    name_en = Column('order_type_name_en', String(300))

    def __repr__(self):
        return '<OrderType {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class OrderAction(Base):
    """
    Действие приказа
    """
    __tablename__ = 'univer_order_action'

    # Идентификатор
    id = Column('order_action_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('order_action_name_ru', String(500))
    name_kz = Column('order_action_name_kz', String(500))
    name_en = Column('order_action_name_en', String(500))

    # Короткое наименование
    short_name_ru = Column('order_action_short_name_ru', String(200))
    short_name_kz = Column('order_action_short_name_kz', String(200))
    short_name_en = Column('order_action_short_name_en', String(200))

    # ???
    value_ru = Column('order_action_value_ru', String(250))
    value_kz = Column('order_action_value_kz', String(250))
    value_en = Column('order_action_value_en', String(250))

    # Тип приказа
    order_type_id = Column(ForeignKey('univer_order_type.order_type_id'))
    order_type = relationship('OrderType')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<OrderAction {} (id={} order_type={} status={})>'.format(self, self.id, self.order_type_id, self.status)

    def __str__(self):
        return self.name_ru


class OrderStudentLink(Base):
    """
    Студент внутри приказа
    """
    __tablename__ = 'univer_order_student_link'

    # Приказ
    order_id = Column(ForeignKey('univer_order.order_id'), primary_key=True)
    order = relationship('Order')

    # Студент
    student_id = Column(ForeignKey(
        'univer_students.students_id'), primary_key=True)
    student = relationship('Student')

    # Данные внутри приказа в XML-формате
    data = Column('osl_data', String)

    def __repr__(self):
        return '<OrderStudentLink {} (order={} student={})>'.format(self, self.order_id, self.student_id)

    def __str__(self):
        return '{} ({})'.format(self.student, self.order)



import unittest
from univer_db.models import EducPlan, EducPlanPos, Cycle, EducLang, Group

from . import Session


class TestEducPlan(unittest.TestCase):
    def test_get_list(self):
        session = Session()
        educ_plans = session.query(EducPlan).filter()
        print("educ_plans: %s" % educ_plans.count())
        self.assertTrue(educ_plans.count())

    def test_get_obj(self):
        session = Session()
        educ_plan = session.query(EducPlan).first()
        self.assertTrue(educ_plan)
        print('educ_plan.min_cred_total: %s' % educ_plan.min_cred_total)


class TestEducPlanPos(unittest.TestCase):
    def test_get_list(self):
        session = Session()
        educ_plan_pos_list = session.query(EducPlanPos).filter()
        print('educ_plan_pos_list: %s' % educ_plan_pos_list.count())
        self.assertTrue(educ_plan_pos_list.count())

    def test_get_obj(self):
        session = Session()
        educ_plan_pos = session.query(EducPlanPos).first()
        self.assertTrue(educ_plan_pos)
        print('educ_plan_pos.credit: %s' % educ_plan_pos.credit)

    def test_get_cycle(self):
        session = Session()
        educ_plan_pos = session.query(EducPlanPos).first()
        print('educ_plan_pos.cycle: %s' % educ_plan_pos.cycle)
        self.assertTrue(educ_plan_pos.cycle)


class TestCycle(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        cycles = session.query(Cycle).filter()
        for cycle in cycles:
            print('cycle: %s' % cycle)
        self.assertTrue(cycles.count())


class TestGroup(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        group = session.query(Group).first()
        print('group: %s' % group)
        print('group.educ_lang: %s' % group.educ_lang)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name="python-univer",
    version="0.9.1",
    author="Nauryzbek Aitbayev",
    author_email="nauryzbek.aitbayev@yu.edu.kz",
    description="Данная библиотека содержить SqlAlchemy ORM для системы Univer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yessenovuniversity/python_univer",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    install_requires = [
        'sqlalchemy',
        'pyodbc',
    ],
    python_requires='>=3.6',
)



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class ControllTypeMarkTypeLink(Base):
    """
    Связь между ControllType и MarkType
    """

    __tablename__ = 'univer_controll_type_mark_type_link'

    # Тип контроля
    controll_type_id = Column(ForeignKey('univer_controll_type.controll_type_id'), primary_key=True)
    controll_type = relationship('ControllType')

    # Тип оценки
    mark_type_id = Column(ForeignKey('univer_mark_type.mark_type_id'), primary_key=True)
    mark_type = relationship('MarkType')

    def __repr__(self):
        return '<ControllTypeMarkTypeLink {} (controll_type_id={} mark_type_id={})>'.format(self, self.controll_type_id, self.mark_type_id)
    
    def __str__(self):
        return '{} - {}'.format(self.controll_type, self.mark_type)